% demo for noncircular PCA
%
%
% Reference:
% X.-L. Li, T. Adali, and M. Anderson, "Noncircular principal component
% analysis and its application to model selection," IEEE Trans. Signal Processing, 
% vol. 59, no. 10, pp. 4516-4528i, Oct. 2011. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
close all

M = 3;      % number of sources
N = 10;      % number of sensors
T = 1000;    % number of samples
snr = 5;    % SNR dB

% generate sources
s = zeros(M,T);
J = sqrt(-1);
s(1,:) = randsrc(1,T,[-1,1;0.5,0.5]);
s(2,:) = randsrc(1,T,[-1+J,1+J,-1-J,1-J;0.25,0.25,0.25,0.25]);
s(3,:) = randsrc(1,T,[-1+J,1+J,-1-J,1-J,-3+J,3+J,-3-J,3-J;0.125,0.125,0.125,0.125,0.125,0.125,0.125,0.125]);

% DOA of the signals
theta1 = 5*pi/180; theta2 = -theta1; theta3 = 0;
for n = 1 : N
    A(n,1) = exp( sqrt(-1)*pi*n*sin(theta1) );
    A(n,2) = exp( sqrt(-1)*pi*n*sin(theta2) );
    A(n,3) = exp( sqrt(-1)*pi*n*sin(theta3) );
end

%fprintf('\nSingular values of A: \n%g, %g, %g\n', svd(A))

x = A*s;
x = awgn(x,snr,'measured');

%[U, U0, dl] = ncPCA( x, M, 3 );
[Mc, Uc, Mnc, pnc, Unc] = order_selection( x );

dist1 = norm(Uc*Uc' - A*inv(A'*A)*A','fro')^2;
fprintf('\nDistance between true signal subspace and signal subspace estimated using circular PCA: \n%g\n', dist1)
dist2 = norm(Unc*Unc' - A*inv(A'*A)*A','fro')^2;
fprintf('\nDistance between true signal subspace and signal subspace estimated using noncircular PCA: \n%g\n', dist2)
